#!/usr/bin/env python

"""
Parsing of vCalendar and iCalendar files.

Copyright (C) 2008 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free
Software Foundation; either version 3 of the License, or (at your option) any
later version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
details.

You should have received a copy of the GNU Lesser General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.

--------

References:

RFC 2445: Internet Calendaring and Scheduling Core Object Specification
          (iCalendar)
          http://rfc.net/rfc2445.html
"""

from vContent import Parser, Reader

try:
    set
except NameError:
    from sets import Set as set

class vCalendarParser(Parser):

    "A parser specifically for vCalendar/iCalendar."

    quoted_parameters = set([
        "ALTREP", "DELEGATED-FROM", "DELEGATED-TO", "DIR", "MEMBER", "SENT-BY"
        ])
    multivalued_parameters = set([
        "DELEGATED-FROM", "DELEGATED-TO", "MEMBER"
        ])
    quoted_types = set(["URI"])

    def decode_quoted_value(self, value):

        "Decode the given 'value', returning a list of decoded values."

        if value[0] == '"' and value[-1] == '"':
            return value[1:-1]
        else:
            return value

    def handleProperty(self, name, parameters, value):

        """
        Record the property with the given 'name', 'parameters' and 'value' as
        part of the current component's children.
        """

        decoded_parameters = {}
        for param_name, param_value in parameters.items():
            if param_name in self.quoted_parameters:
                param_value = self.decode_quoted_value(param_value)
                separator = '","'
            else:
                separator = ","
            if param_name in self.multivalued_parameters:
                param_value = param_value.split(separator)
            decoded_parameters[param_name] = param_value

        return Parser.handleProperty(self, name, decoded_parameters, value)

# Public functions.

def parse(f, non_standard_newline=0):

    """
    Parse the resource data found through the use of the file object 'f', which
    should provide Unicode data, and put the resource information in the given
    'store'. (The codecs module can be used to open files or to wrap streams in
    order to provide Unicode data.)

    The optional 'non_standard_newline' can be set to a true value (unlike the
    default) in order to attempt to process files with CR as the end of line
    character.

    As a result of parsing the resource, the root node of the imported resource
    is returned.
    """

    reader = Reader(f, non_standard_newline=non_standard_newline)
    parser = vCalendarParser()
    return parser.parse(reader)

# vim: tabstop=4 expandtab shiftwidth=4
